/*
	File:		MPEG2Receiver.h

    Synopsis: This is the header file for the MPEG2Receiver class. 
 
	Copyright: 	 Copyright 2001-2003 Apple Computer, Inc. All rights reserved.

	Written by: ayanowitz

 Disclaimer:	IMPORTANT:  This Apple software is supplied to you by Apple Computer, Inc.
 ("Apple") in consideration of your agreement to the following terms, and your
 use, installation, modification or redistribution of this Apple software
 constitutes acceptance of these terms.  If you do not agree with these terms,
 please do not use, install, modify or redistribute this Apple software.

 In consideration of your agreement to abide by the following terms, and subject
 to these terms, Apple grants you a personal, non-exclusive license, under Apples
 copyrights in this original Apple software (the "Apple Software"), to use,
 reproduce, modify and redistribute the Apple Software, with or without
 modifications, in source and/or binary forms; provided that if you redistribute
 the Apple Software in its entirety and without modifications, you must retain
 this notice and the following text and disclaimers in all such redistributions of
 the Apple Software.  Neither the name, trademarks, service marks or logos of
 Apple Computer, Inc. may be used to endorse or promote products derived from the
 Apple Software without specific prior written permission from Apple.  Except as
 expressly stated in this notice, no other rights or licenses, express or implied,
 are granted by Apple herein, including but not limited to any patent rights that
 may be infringed by your derivative works or by other works in which the Apple
 Software may be incorporated.

 The Apple Software is provided by Apple on an "AS IS" basis.  APPLE MAKES NO
 WARRANTIES, EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION THE IMPLIED
 WARRANTIES OF NON-INFRINGEMENT, MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 PURPOSE, REGARDING THE APPLE SOFTWARE OR ITS USE AND OPERATION ALONE OR IN
 COMBINATION WITH YOUR PRODUCTS.

 IN NO EVENT SHALL APPLE BE LIABLE FOR ANY SPECIAL, INDIRECT, INCIDENTAL OR
 CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE
 GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 ARISING IN ANY WAY OUT OF THE USE, REPRODUCTION, MODIFICATION AND/OR DISTRIBUTION
 OF THE APPLE SOFTWARE, HOWEVER CAUSED AND WHETHER UNDER THEORY OF CONTRACT, TORT
 (INCLUDING NEGLIGENCE), STRICT LIABILITY OR OTHERWISE, EVEN IF APPLE HAS BEEN
 ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

*/

#ifndef __FIREWIREMPEG_MPEG2RECEIVER__
#define __FIREWIREMPEG_MPEG2RECEIVER__

// enum for this module's messages
enum
{
	// Note: Both these messages result in receveiver stopping
	kMpeg2ReceiverReceivedBadPacket,
	kMpeg2ReceiverDCLOverrun,

	kMpeg2ReceiverAllocateIsochPort,
	kMpeg2ReceiverReleaseIsochPort
};

// enum for transport states
enum
{
	kMpeg2ReceiverTransportStopped,
	kMpeg2ReceiverTransportRecording
};

// Default values for receive segment size and number of segments.
// These can be overriden in the receiver class constructor
#define kNumReceiveSegments	3
#define kCyclesPerReceiveSegment 2000

// Other defines
#define kMaxNumReceivePacketsPerCycle 4
#define kMPEG2ReceiveBufferSize ((kMaxNumReceivePacketsPerCycle*192)+16)

// Structure containing vars for each receive segment
typedef struct MPEGReceiveSegmentStruct
{
	DCLLabelPtr pSegmentLabelDCL;
	DCLJumpPtr pSegmentJumpDCL;
}MPEGReceiveSegment, *MPEGReceiveSegmentPtr;

// Function prototype for data push callback.
typedef IOReturn (*DataPushProc) (UInt32 tsPacketCount, UInt32 **ppBuf);

// Function prototype for message callback.
typedef void (*MPEG2ReceiverMessageProc) (UInt32 msg, UInt32 param1, UInt32 param2);

//
// The MPEG2 Receiver Class Declaration
//
class MPEG2Receiver
{

public:
	// Constructor
    MPEG2Receiver(StringLogger *stringLogger = nil,
				  IOFireWireLibNubRef nubInterface = nil,
				  unsigned int cyclesPerSegment = kCyclesPerReceiveSegment,
				  unsigned int numSegments = kNumReceiveSegments,
				  bool doIRMAllocations = false);

    // Destructor
    ~MPEG2Receiver();

    // Function to setup all the isoc stuff
    IOReturn setupIsocReceiver(void);

	// Set the isoch receive channel
	IOReturn setReceiveIsochChannel(unsigned int chan);

	// Set the isoch receive speed -
	// Note: Only important to set if doIRMAllocations is enabled in class constructor
	IOReturn setReceiveIsochSpeed(IOFWSpeed speed);

	// Start/Stop routines
	IOReturn startReceive(void);
	IOReturn stopReceive(void);

	// Function to install a handler for receiving data packets
	IOReturn registerDataPushCallback(DataPushProc handler);

	// Function to install a handler for receiving messages
	IOReturn registerMessageCallback(MPEG2ReceiverMessageProc handler);

	// Publically visible vars
    double mpegDataRate;	// Users should treat as read-only for status
	unsigned int transportState;

	// A reference to the current run loop for isoch callbacks
	CFRunLoopRef runLoopRef;
	
private:

	// Interface pointers

	IOFireWireLibNubRef nodeNubInterface;
	IOFireWireLibDCLCommandPoolRef dclCommandPool;
	IOFireWireLibRemoteIsochPortRef remoteIsocPort ;
	IOFireWireLibLocalIsochPortRef localIsocPort;
	IOFireWireLibIsochChannelRef isochChannel;
	StringLogger *logger;

	// Call to fixup jump targets in receive dcl
	void fixupDCLJumpTargets(void);
	
	// Class Variables
	DCLCommandStruct *pFirstDCL;
	DCLCommandPtr *updateDCLList;
	unsigned int receiveChannel;
	IOFWSpeed receiveSpeed;
    UInt8 *pReceiveBuffer;
	MPEGReceiveSegment *receiveSegmentInfo;
	DCLLabelPtr pDCLOverrunLabel;
	UInt32 currentSegment;
	unsigned int isochCyclesPerSegment;
	unsigned int isochSegments;
	unsigned int dclCommandPoolSize;
	bool doIRM;
	bool noLogger;
	bool noLocalNodeInterface;
	unsigned int dclVMBufferSize;

	// Registered Handler functions
	DataPushProc packetPush;
	MPEG2ReceiverMessageProc messageProc;

public:
	// Callbacks for remote isoch port object
	IOReturn RemotePort_GetSupported(
										 IOFireWireLibIsochPortRef interface,
										 IOFWSpeed *outMaxSpeed,
										 UInt64 *outChanSupported);
	IOReturn RemotePort_AllocatePort(
										 IOFireWireLibIsochPortRef interface,
										 IOFWSpeed maxSpeed,
										 UInt32 channel);
	IOReturn RemotePort_ReleasePort(
										IOFireWireLibIsochPortRef interface);
	IOReturn RemotePort_Start(
								  IOFireWireLibIsochPortRef interface);
	IOReturn RemotePort_Stop(
								 IOFireWireLibIsochPortRef interface);

	// Callbacks for DCL
	void MPEG2ReceiveDCLCallback(void);
	void MPEG2ReceiveOverrunDCLCallback(void);
};

#endif // __FIREWIREMPEG_MPEG2RECEIVER__
